#include <vector>

using namespace std;

#include <saotrace/io/fits/fits.h>

namespace FITS = SAOTrace::IO::FITS;

#include <gtest/gtest.h>

typedef vector<long> Shape;

#define SHAPE(id)				\
    Shape id( id ## _i, id ## _i + sizeof(id ## _i) / sizeof(int));

TEST( CmpShape, Same ) {

    int a_i[] = { 1, 2, 3 };
    SHAPE(a);

    ASSERT_TRUE( FITS::cmp_shape( a, a ) ) << "{1,2,3} {1,2,3}";;

}

TEST( CmpShape, FullVsEmpty ) {

    int a_i[] = { 1, 2, 3 };
    SHAPE(a);
    Shape b;

    ASSERT_FALSE( FITS::cmp_shape( a, b ) ) << "{1,2,3} {}";
    ASSERT_FALSE( FITS::cmp_shape( b, a ) ) << "{} {1,2,3}";

}

TEST( CmpShape, Similar ) {

    {
	int a_i[] = { 1, 2, 1 };
	SHAPE(a);

	int b_i[] = { 2 };
	SHAPE(b);

	ASSERT_TRUE( FITS::cmp_shape( a, b ) ) << "{1,2,1} {2}";

	ASSERT_TRUE( FITS::cmp_shape( b, a ) ) << "{2} {1,2,1}";


    }

    {
	int a_i[] = { 2, 1, 2, 3, 1 };
	SHAPE(a);

	int b_i[] = { 2, 2, 3 };
	SHAPE(b);

	ASSERT_TRUE( FITS::cmp_shape( a, b ) ) << "{ 2, 1, 2, 3, 1 } { 2, 2, 3 }";

	ASSERT_TRUE( FITS::cmp_shape( b, a ) ) << "{ 2, 2, 3 } { 2, 1, 2, 3, 1 }";

    }


}

TEST( CmpShape, DisSimilar ) {

    {
	int a_i[] = { 3, 1, 2, 1 };
	SHAPE(a);

	int b_i[] = { 2 };
	SHAPE(b);

	ASSERT_FALSE( FITS::cmp_shape( a, b ) ) << "{3,1,2,1} {2}";

	ASSERT_FALSE( FITS::cmp_shape( b, a ) ) << "{2} {3,1,2,1}";


    }

    {
	int a_i[] = { 5, 1, 2, 3, 1 };
	SHAPE(a);

	int b_i[] = { 2, 2, 3 };
	SHAPE(b);

	ASSERT_FALSE( FITS::cmp_shape( a, b ) ) << "{ 5, 1, 2, 3, 1 } { 2, 2, 3 }";

	ASSERT_FALSE( FITS::cmp_shape( b, a ) ) << "{ 2, 2, 3 } { 5, 1, 2, 3, 1 }";

    }


}
