#include <string>
#include <vector>

#include <fstream>
#include <sstream>
#include <numeric>

#include <boost/format.hpp>

#include <fitsio.h>

#include "cfitsio_exception.h"

using namespace std;


#include "column_fits.h"

#include <boost/serialization/vector.hpp>
#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>


namespace FITSTest {

    int Column::col_counter(0);

    Column::Column( const char* namev, const char* form_code, int i, int j, int k) {

	name = namev;

	stringstream dim_s;

	// horrible hack to create extents
	if (i) {
	    extent.push_back(i);
	    dim_s << i;
	    if (j) {
		extent.push_back(j);
		dim_s << "," << j;
		if (k) {
		    dim_s << "," << k;
		    extent.push_back(k);
		}
	    }
	}

	repeat = accumulate( extent.begin(), extent.end(), 1, multiplies<int>() );
	form = boost::str( boost::format( "%1%%2%") % repeat % form_code );

	colno = ++col_counter;
	dimkey = boost::str( boost::format( "TDIM%1%" ) % colno );
	dimstr = "(" + dim_s.str() + ")";
    }


    int
    Column::write( fitsfile* fptr, int nrows, int *status ) {

	const long nelements = nrows * repeat;

	typedef vector<int> Value;
	Value value(nelements);
	Value::iterator ival;
	Value::iterator begin = value.begin();;
	Value::iterator end  = value.end();

	minval = colno;
	maxval = minval - 1;

	// fabricate some values
	for ( ival = begin; ival != end ; ++ival )
	    *ival = ++maxval;

	// write the TDIMn for the cell shape
	if ( fits_write_key( fptr,
			     TSTRING,
			     dimkey.c_str(),
			     static_cast<void*>(const_cast<char*>(dimstr.c_str())),
			     "",
			     status ) )
	    throw( cfitsioException( "error writing tdim" ) );

	if ( fits_write_col( fptr, TINT,
			     colno, // column
			     1,     // firstrow
			     1,     // first element
			     nelements,
			     &value[0],
			     status ) )
	    throw( cfitsioException( "error writing column" ) );

	return *status;
    }

    void
    Column::read_meta ( const string&filename, MetaData& meta_data, MetaMap& meta_map) {

	string archive( filename + ".meta" );
	std::ifstream ifs( archive.c_str() );
	boost::archive::text_iarchive ia( ifs );
	ia >> meta_data;

	for( MetaData::iterator icol = meta_data.begin() ; icol != meta_data.end() ; ++ icol )
	    meta_map.insert( make_pair( icol->name, &icol[0] ) );
    }


}
