#include <string>
#include <vector>
#include <map>
#include <iostream>
#include <fstream>
#include <sstream>
#include <numeric>
#include <boost/format.hpp>

#include <boost/serialization/vector.hpp>
#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>


using namespace std;

#include <fitsio.h>

#include "cfitsio_exception.h"
#include "column_fits.h"

using namespace FITSTest;

int main( int argc, char *argv[] ) {

    long nrows;

    int status = 0;
    fitsfile* fptr;

    if (argc < 2 )
	throw( Exception("must specify output file name") );

    string fitsfile( argv[1] );

    // define some columns

    vector<Column> columns;
    vector<Column>::iterator icol;
    columns.push_back( Column( "vdouble", "D", 1    ) );
    columns.push_back( Column( "vfloat",  "E", 1, 1 ) );
    columns.push_back( Column( "vshort",  "I", 2, 1 ) );
    columns.push_back( Column( "vint",    "J", 2, 2 ) );
    columns.push_back( Column( "vlong",   "K", 1, 2 ) );


    // create structures for CFITSIO calls
    {
	const char* * name = new const char*[ columns.size() ];
	const char* * form = new const char*[ columns.size() ];

	for(  icol = columns.begin() ; icol != columns.end() ; ++icol ) {

	    long idx = icol->colno - 1;
	    name[idx] = icol->name.c_str();
	    form[idx] = icol->form.c_str();
	}

	string fitsfile_expr = string("!") + fitsfile;
	if ( fits_create_file( &fptr, fitsfile_expr.c_str(), &status ) )
	    throw( cfitsioException( fitsfile + ":error creating" ) );

	if ( fits_create_tbl( fptr,
			      BINARY_TBL,
			      0,
			      static_cast<int>(columns.size()),
			      const_cast<char**>(name),
			      const_cast<char**>(form),
			      NULL, // unit
			      const_cast<char*> ("vtable"),
			      &status ) )
	    throw( cfitsioException( fitsfile + ":error creating table ") );

	// make sure the CFITSIO iterator is called multiple times by writing more rows
	// than its optimal chunk size
	if ( fits_get_rowsize( fptr, &nrows, &status ) )
	    throw( cfitsioException( fitsfile + ":error getting optimal number of rows") );
	nrows *= 3;

	delete[] name;
	delete[] form;
    }

    for( icol = columns.begin(); icol != columns.end() ; ++icol )
	icol->write( fptr, nrows, &status );


    if ( fits_close_file( fptr, &status ) )
	throw( cfitsioException( fitsfile + ":error closing file" ) );

    string archive(fitsfile + ".meta");
    std::ofstream ofs( archive.c_str() );
    boost::archive::text_oarchive oa( ofs );
    oa << columns;

    return 0;
}
