#include <stdlib.h>
#include <string>
#include <algorithm>

using namespace std;

#include <gtest/gtest.h>

#include <Exception/Exception.h>

#include <saotrace/io/fits/fits.h>

namespace FITS = SAOTrace::IO::FITS;

#include "args.h"

#include "column_fits.h"

using namespace FITSTest;

// ------------------------------------------------------------------------ //

// command line arguments

class Args : public TestArgsBase {

public:

    TCLAP::ValueArg<std::string> fitsfile;

    Args( ) :
	TestArgsBase( "read_columns" ),
	fitsfile( "f", "fitsfile", "FITS file to read", true, "", "filename", cmd ) {}

};

static Args args;

// ------------------------------------------------------------------------ //

// Meta information about the columns in the input FITS file

static Column::MetaData column_meta;
static Column::MetaMap column_meta_map;

TEST(HeaderColumns,Exists) {


    FITS::TableFH fh( args.fitsfile.getValue() );

    // to test that the FITSIO status flag is properly handled, try
    // a mix of existing and non-existing columns.

    // make sure that we use a column that doesn't exist.
    ASSERT_EQ( column_meta_map.find( "doesnotexist" ), column_meta_map.end() );

    ASSERT_TRUE( fh.has_column( "vdouble" ) );

    ASSERT_FALSE( fh.has_column( "doesnotexist" ) );

    ASSERT_TRUE( fh.has_column( "vfloat" ) );

    ASSERT_FALSE( fh.has_column( "doesnotexist" ) );

}



// ------------------------------------------------------------------------ //

GTEST_API_ int main(int argc, char **argv) {

  args.parse( &argc, argv );

  Column::read_meta( args.fitsfile.getValue(), column_meta, column_meta_map);

  testing::InitGoogleTest(&argc, argv);

  return RUN_ALL_TESTS();
}
